/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/SPFileChooser.java,v 1.2 2001/07/17 04:00:46 msmith Exp $
 * $Revision: 1.2 $
 * $Date: 2001/07/17 04:00:46 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser;

import java.io.File;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.filechooser.*;

import org.apache.webdav.lib.*;
import org.apache.webdav.ui.filechooser.dir.*;
import org.apache.webdav.ui.filechooser.plafui.*;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpClient;

/**
 * Title:        SPFileChooser.java
 * Description:  A Web File Chooser.
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPFileChooser extends JFileChooser {

    private static SPDirectoryService directoryService;
    private boolean filterNaming = true;

    public static SPFileChooser createWebFileChooser(HttpClient client,
            String rootUrlPath) throws Exception {

        try {
            directoryService =
                    new SPDirectoryService(client, rootUrlPath);
            SPFileSystemView fileSystemView =
		    new SPFileSystemView(directoryService);
	    SPFileChooser fileChooser = new SPFileChooser(fileSystemView);
            return fileChooser;

        } catch (Exception e) {
            throw e;
        }
    }

    public static SPFileChooser createWebFileChooser(SPFile currentDirectory)
	throws Exception {
	SPDirectoryService directoryService =
	    currentDirectory.getDirectoryService();

	SPFile oldHome = directoryService.getHomeDirectory();
	directoryService.resetHomeDirectory();

	SPFileSystemView fsv =
	    new SPFileSystemView(directoryService);

	SPFileChooser fileChooser = new SPFileChooser(fsv);


	fileChooser.setHomeDirectory(oldHome.getResourceUrlPath());
	fileChooser.gotoDirectory(currentDirectory);

	return fileChooser;
    }

    private SPFileChooser(FileSystemView fsv) {
	super(fsv);
	uiUpdate();
        setFileView(new SPFileView());
        setFileSelectionMode(JFileChooser.FILES_ONLY);
    }

    private SPFileChooser(SPFile currentDirectory, FileSystemView fsv)  {
	this(fsv);
    }

    public SPDirectoryService getDirectoryService() {
        return directoryService;
    }

    public void addFileFilter(String filterDescription) {
        if (isFilterSyntaxCorrect(filterDescription)) {
            FileFilter aFilter =
                    new SPFileFilter(filterDescription);
            addChoosableFileFilter(aFilter);
        }
    }

    public void setFileFilter(String filterDescription) {
        if (isFilterSyntaxCorrect(filterDescription)) {
            FileFilter aFilter =
                    new SPFileFilter(filterDescription);
            setFileFilter(aFilter);
        }
    }

    /* if filterNaming is true, the first filename extension
     * listed in the filter description of the current file filter
     * is appended to the selected file's name.
     */
    public void setFilterNaming(boolean b) {
	filterNaming = b;
    }

    public boolean isFilterNaming()  {
        return filterNaming;
    }

    public void setHomeDirectory(String homeUrlPath) throws Exception {
        //homeUrlPath must point to a collection
        if (! homeUrlPath.endsWith("/")) {
	    homeUrlPath = homeUrlPath+"/";
	}

	if (getHomeDirectory().getResourceUrlPath().equals(homeUrlPath))  {
	    return;
	}

	SPFile root = directoryService.getRoot();

        if (homeUrlPath.indexOf(root.getResourceUrlPath()) != 0)  {
            throw (new HttpException("Bad URL Path: "+homeUrlPath
	            +" NOT Under Root URL Path"));
        }

	/* Pending: Currently a file path is equivalent to its url path */
	String homeFilePath = homeUrlPath.substring(
	    root.getResourceUrlPath().length());

	if (File.separatorChar != '/') {
	    homeFilePath = homeFilePath.replace('/', '\\');
	}

	directoryService.setHomeDirectory(
	    root.getAbsolutePath()+File.separator+homeFilePath, homeUrlPath);
    }

    public SPFile getHomeDirectory()  {
        return (SPFile) directoryService.getHomeDirectory();
    }

    public void gotoDirectory(SPFile aFile)  {
	SPFile currentDir = (!aFile.isTraversable())
			        ? (SPFile) aFile.getParentFile()
				: aFile;
        aFile.refreshContent();
        setCurrentDirectory(aFile);
    } // gotoDirectory()

    public SPFile getSelectedSPFile() {
	return (SPFile) getSelectedFile();
    }

    public Color getTextColor(File f) {
	Color textColor = Color.black; //default color
	SPFileView fileView = (SPFileView) getFileView();

	if ((f != null) && (fileView != null)) {
		textColor = fileView.getTextColor(f);
        }

	return textColor;
    }


    private void uiUpdate() {
        FileChooserUI ui;
        if (System.getProperty("os.name").indexOf("Windows")>=0)  {
	    ui = new SPWindowsFileChooserUI(this);
        } else  {
	    ui = new SPMetalFileChooserUI(this);
        }

        setUI(ui);
    }

    private boolean isFilterSyntaxCorrect(String filterDescription) {
        boolean   isCorrect = false;
        int openingBrace = filterDescription.indexOf("(");
        int closingBrace = filterDescription.indexOf(")");

        if ((closingBrace > openingBrace) && (openingBrace >= 0)) {
            //there must me opening and closing braces
            String extList = filterDescription.substring(openingBrace+1, closingBrace);
            extList = extList.trim();

            if ( (extList.length() >= 2) && (extList.indexOf(".") >= 0) ) {
                //at least 2 characters and there is a "."; for example: .x
                isCorrect = true;
            }
        }
        return isCorrect;
      }

}// End of SPFileChooser class









