/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/dir/SPWebFolder.java,v 1.4 2001/10/16 23:44:45 msmith Exp $
 * $Revision: 1.4 $
 * $Date: 2001/10/16 23:44:45 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser.dir;

import java.util.*;
import java.io.*;

import org.apache.webdav.lib.*;
import org.apache.webdav.ui.lib.methods.SPPropFindMethod;
import org.apache.webdav.ui.lib.properties.*;
import org.apache.webdav.ui.util.SPUtils;

import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.HttpClient;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Title:        SPWebFolder.java
 * Description:  A Collection Resource Node
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPWebFolder extends SPResourceNode {

  protected List children = null;

  public SPWebFolder(HttpClient client, String urlPath) {
    this(client, urlPath, SPUtils.getFileNameOfUrl(urlPath));
  }

  public SPWebFolder(HttpClient client, String urlPath, String name) {
    this.client = client;

    //urlPath must point to a collection
    urlPath = (urlPath.endsWith("/")) ? urlPath : urlPath+"/";
    this.resourceUrlPath = urlPath;
    this.name = name;
  }
/*
  public SPWebFolder(HttpClient client, String urlPath,
        boolean readAccess, boolean writeAccess) {
    this(client, urlPath, SPUtils.getFileNameOfUrl(urlPath),
         readAccess, writeAccess);
  }

  public SPWebFolder(HttpClient client, String urlPath, String name,
        boolean readAccess, boolean writeAccess) {

    this.client = client;

    //urlPath must point to a collection
    urlPath = (urlPath.endsWith("/")) ? urlPath : urlPath+"/";
    this.resourceUrlPath = urlPath;
    this.name = name;
    this.readAccess = readAccess;
    this.writeAccess = writeAccess;
    accessPrivilegeChecked = true;
  }
*/

  public List getChildren()  {
    if (children == null) {
	refreshContent();
    }
    return children;
  }

  public void refreshContent()  {
      try {
        init();
      } catch (Exception e) {
        e.printStackTrace();
      }
  }

  public void init() throws Exception  {
    SPPropFindMethod propFindMethod = new SPPropFindMethod(client);
    propFindMethod.setPath(resourceUrlPath);
    propFindMethod.execute();

    int statusCode = propFindMethod.getStatusCode();

    if (statusCode<200 || statusCode >299)  {
      final String title = "Remote Failure";
      final String msg = "Unable to access a remote directory ("+statusCode+")";
      SPUtils.showMessageDialog(title, msg);
    }

    children = new ArrayList();
    Enumeration enum = propFindMethod.getAllResponseURLs();

    while (enum.hasMoreElements()){
      String url = (String) enum.nextElement();
      Property p = (Property)propFindMethod.getProperty("current-user-privilege-set", url);
	  //System.err.println("current-user-privilege-set is "+p.getClass().getName());
      CurrentUserPrivilegeSetProperty userPrivilege =
//	    (CurrentUserPrivilegeSetProperty)
 //            propFindMethod.getProperty("current-user-privilege-set", url);
	(CurrentUserPrivilegeSetProperty)p;

      boolean readAccess = userPrivilege.hasReadAccess();
      boolean writeAccess = userPrivilege.hasWriteAccess();

      LockdiscoveryProperty lockdiscoveryProp =
	    (LockdiscoveryProperty)
	     propFindMethod.getProperty("lockdiscovery", url);

      boolean readOnly=false;
      if (lockdiscoveryProp != null)  { //is being locked
	  String userName = client.getCredentials().getUserName();
	  if (lockdiscoveryProp.getLockToken(userName) == null)  {
	      // is locked by others
	      readOnly = true;
	  }
      }

      SPResourceNode newChild;

      // Pending: Waiting for the proper setting of "displayname" property
      // in the server

      if( ! url.endsWith(resourceUrlPath) ) {
        //if not this SPWebFolder's url
        //create a SPWebFolder or SPWebFile child according to its type

        if (propFindMethod.isCollection(url))  {
          newChild = new SPWebFolder(client, url);
        } else  {
          newChild = new SPWebFile(client, url);
        }

	newChild.setPrivilege(readAccess, writeAccess);
	newChild.setReadOnlyCached(readOnly);
	newChild.setParent(this);
        children.add(newChild);
      } else  {
        this.readAccess = readAccess;
        this.writeAccess = writeAccess;
        accessPrivilegeChecked = true;
      }// if(! url.endsWidth...)
    }// while (enum...)
  }// init()

  public void addWebFile(SPWebFile aWebFile) {
    aWebFile.setParent(this);
    children.add(aWebFile);
  }

  public boolean isWebFolder()  {
    return true;
  }

  public boolean isAcceptable(String aFilterDescription) {
    return true; //always display a web folder in file chooser
  }

  public int getType()  {
    return SPResourceNode.COLLECTION_TYPE;
  }

}// End of SPWebFolder class









