/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/lib/properties/LockdiscoveryProperty.java,v 1.1 2001/07/17 04:00:53 msmith Exp $
 * $Revision: 1.1 $
 * $Date: 2001/07/17 04:00:53 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.lib.properties;

import java.util.*;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import org.apache.webdav.lib.Property;
import org.apache.util.DOMUtils;

/**
 * Title:        LockdiscoveryProperty.java
 * Description:
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class LockdiscoveryProperty extends SPProperty {

  public LockdiscoveryProperty(Property property) {
    super(property);
  }

  public ArrayList getActiveLockOwners() {
    ArrayList theActiveLockOwners = new ArrayList();

    NodeList ownerList = property.getElement().getElementsByTagName("owner");
    for (int i=0; i < ownerList.getLength(); i++) {
      Element ownerElement = (Element) ownerList.item(i);
      String activeLockOwner =  DOMUtils.getTextValue(ownerElement);
      theActiveLockOwners.add(activeLockOwner);
    }//for(i)

    return theActiveLockOwners;
  } //getActiveLockOwners()

  public NodeList getActiveLocks() {
    return property.getElement().getElementsByTagName("activelock");
  }

  public Element getActiveLock(String ownerName)  {
    String owner = "/users/"+ownerName;

    Element theActiveLock = null;

    NodeList activeLockList = getActiveLocks();
    boolean notFound = true;

    for (int i = 0; notFound && i < activeLockList.getLength(); i++ ) {
      Element activeLockElement = (Element) activeLockList.item(i);
      NodeList ownerList = activeLockElement.getElementsByTagName("owner");

      if (ownerList.getLength() == 1) { //if exists
        Element ownerElement = (Element) ownerList.item(0);

        String activeLockOwner = DOMUtils.getTextValue(ownerElement);

        notFound = (owner.indexOf(activeLockOwner)>=0) ? false : true;
        theActiveLock = (notFound) ? null : activeLockElement;

      } //if (ownerList.getLength() == 1);
    }//for(i)

    return theActiveLock;
  }//getAcctiveLock()

  public String getLockToken(String ownerName)  {
    String theLockToken=null;
    Element activeLock = getActiveLock(ownerName);
    if (activeLock != null) {
      NodeList list = activeLock.getElementsByTagName("locktoken");

      if (list.getLength() == 1) { //if exists
         Element locktoken = (Element) list.item(0);
         NodeList list2 = locktoken.getElementsByTagName("href");
         if (list2.getLength() == 1) {
            theLockToken = DOMUtils.getTextValue(list2.item(0));
         }
      }
    }//if (activeLock != null);

    return theLockToken;
  }//getLockToken()

  public long getTimeout(String ownerName)  {
    long theTimeout=0;
    Element activeLockElement = getActiveLock(ownerName);

    if (activeLockElement != null)  {
      NodeList list = activeLockElement.getElementsByTagName("timeout");
      if (list.getLength() == 1) { //if exists
        String timeoutString = DOMUtils.getTextValue(list.item(0));
        int signIndex = timeoutString.indexOf('-');
        theTimeout = Long.parseLong(timeoutString.substring(signIndex+1));
      }
    }

    return theTimeout;
  }

  public String getPropertyAsString() {
    String owners="";

    ArrayList ownerList = getActiveLockOwners();
    for (Iterator i = ownerList.iterator(); i.hasNext();) {
      owners += (String) i.next()+" | ";
    }

    return (" = owners: "+owners);
  }

}//End of LockdiscoveryProperty class











