/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/util/SPUtils.java,v 1.8 2001/10/16 23:44:45 msmith Exp $
 * $Revision: 1.8 $
 * $Date: 2001/10/16 23:44:45 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.util;

import javax.swing.JOptionPane;
import java.util.*;

import org.apache.webdav.ui.lib.methods.*;
import org.apache.webdav.ui.filechooser.dir.*;
import org.apache.webdav.ui.lib.properties.*;
import org.apache.webdav.lib.properties.ResourceTypeProperty;
import org.apache.webdav.lib.methods.DepthSupport;
import org.apache.webdav.lib.Property;

import org.apache.commons.httpclient.*;

/**
 * Title:        SPUtils.java
 * Description:  Provides some convenient utility functions
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPUtils {

  public static String getFileNameOfUrl(String urlPath)  {
    String n = new String(urlPath);

    if (n.indexOf("\\") >= 0)  {
	return n;
    }

    //extract the last part as file name
    if (n.endsWith("/"))  {
      n = n.substring(0,n.length()-1);
    }

    return n.substring(n.lastIndexOf("/")+1);
  }

  /*
   * Returns the parent path of the given urlPath.
   * The parent path ends with "/" character.
   */
  public static String getParentUrl(String urlPath) {
    String theString = null;

    if (! urlPath.startsWith("/"))  {
	urlPath = "/"+urlPath;
    }

    if (urlPath.length() > 1) {
	theString = urlPath;
	if (theString.endsWith("/"))  {
	    theString = theString.substring(0, theString.length()-1);
	}
	theString = theString.substring(0, theString.lastIndexOf("/")+1);
    }

    return theString;
  }

  public static String convertToClassName(String propertyName) {
      //Remove the namespace and any dash character out of propertyName
      //and change its first character to uppercase.
      String theResult="";

      //First namespace
      int colonIndex = propertyName.indexOf(':');
      propertyName = (colonIndex >= 0)
                     ? propertyName.substring(colonIndex+1)
                     : propertyName;

      propertyName = (propertyName.substring(0,1)).toUpperCase()+
                propertyName.substring(1)+"Property";

      //Then dash character
      StringTokenizer stoken = new StringTokenizer(propertyName, "-");
      while (stoken.hasMoreTokens())  {
        String s = stoken.nextToken();
        theResult += (s.substring(0,1)).toUpperCase() + s.substring(1);
      }
      return theResult;
  }

  /*
   * Get the extension of a file.
   */
  public static String getExtension(String filename) {
    String ext = null;
    int i = filename.lastIndexOf('.');

    if (i > 0 &&  i < filename.length() - 1) {
       ext = filename.substring(i+1).toLowerCase();
    }
    return ext;
  }

  /*
   * Show A Message Dialog with Exception
   */
  public static void showMessageDialog(String title, String message)
                                       throws HttpException {
      JOptionPane.showMessageDialog(null, message,
                title, JOptionPane.ERROR_MESSAGE);

      throw new HttpException(message);
  }

  /*
   * Whether a resource has a Read Only permission.
   *
   * A node is read only if the resource is being locked
   * by OTHER users.
   *
   */
  public static boolean isReadOnly(SPResourceNode resource) {
    boolean lockedByOthers = false;

    LockdiscoveryProperty lockdiscoveryProp =
        (LockdiscoveryProperty) getProperty(resource, "lockdiscovery");

    if (lockdiscoveryProp != null) { // is being locked
	String userName = resource.getClient().getCredentials().getUserName();
        if (lockdiscoveryProp.getLockToken(userName) == null)  {
	    // is being locked by other users
	    lockedByOthers = true;
	}
    }

    return lockedByOthers;
  } // isReadOnly()

  /*
   * Whether a resource is locked
   */
  public static boolean isLocked(SPResourceNode resource) {
    SPProperty lockdiscoveryProp = getProperty(resource, "lockdiscovery");
    boolean locked = (lockdiscoveryProp == null) ? false : true;
    return locked;
  }

  /*
   * Lock a resource for SPResourceNode.DEFAULT_TIMEOUT seconds.
   * Refresh the lock automatically if the resource has already been locked.
   *
   * Returns the resulting lockToken if successful and 'null' otherwise
   */
  public static String lock(SPResourceNode resource)  {
    return lock(resource, SPResourceNode.DEFAULT_TIMEOUT);
  }

  /*
   * Lock a resource for 'timeout' seconds.
   * Refresh the lock automatically if the resource has already been locked.
   *
   * Returns the resulting lockToken if successful and 'null' otherwise
   */
  public static String lock(SPResourceNode resource, long timeout)  {
    String refreshOpaqueToken, lockToken = null;
    SPLockMethod lockMethod = new SPLockMethod(resource, timeout);
    return lock(lockMethod);
  }

  /*
   * Lock a resource specified by an authenticated client and a url
   * for SPResourceNode.DEFAULT_TIMEOUT seconds.
   * Refresh the lock automatically if the resource has already been locked.
   *
   * Returns the resulting lockToken if successful and 'null' otherwise
   */
  public static String lock(HttpClient client, String urlPath)  {
    return lock(client, urlPath, SPResourceNode.DEFAULT_TIMEOUT);
  }

  /*
   * Lock a resource specified by an authenticated client and a url
   * for 'timeout' seconds.
   * Refresh the lock automatically if the resource has already been locked.
   *
   * Returns the resulting lockToken if successful and 'null' otherwise
   */
  public static String lock(HttpClient client, String urlPath, long timeout)  {
    SPLockMethod lockMethod = new SPLockMethod(client, urlPath, timeout);
    return lock(lockMethod);
  }

  /*
   * Lock method execution.
   * Refresh the lock automatically if the resource has already been locked.
   *
   * Returns the resulting lockToken if successful and 'null' otherwise
   */
  public static String lock(SPLockMethod lockMethod)  {
    String lockToken = null;
    try {
      lockMethod = (SPLockMethod) lockMethod.execute();
      int statusCode = lockMethod.getStatusCode();

      if (statusCode == HttpStatus.SC_OK) {
        lockToken = lockMethod.getLockToken();
      } else if (statusCode == HttpStatus.SC_LOCKED)  {
        final String title = "Locking Failure";
        final String msg = "Unable to perform locking.\n"+
                           "The resource is being locked by "+
                           "another user ("+statusCode+").\n"+
                         "Resource Path = "+lockMethod.getPath();
        JOptionPane.showMessageDialog(null, msg, title, JOptionPane.ERROR_MESSAGE);

      } else {
        final String title = "Locking Failure";
        final String msg = "Unable to perform locking("+statusCode+").\n"+
                         "Resource Path = "+lockMethod.getPath();
        showMessageDialog(title, msg);
      }
    } catch(Exception e)  {
      //do nothing
      e.printStackTrace();
    }

    return lockToken;
  } //lock()

  /*
   * Unlock a resource.
   *
   */
  public static boolean unlock(SPResourceNode resource)  {
    boolean theResultFlag = false;
    SPUnlockMethod unlockMethod = new SPUnlockMethod(resource);
    try {
      unlockMethod.execute();
      int statusCode = unlockMethod.getStatusCode();

      System.out.println("Unlock status "+statusCode);

      if (statusCode != HttpStatus.SC_NO_CONTENT) {
        final String title = "Unlocking Failure";
        final String msg = "Unable to unlock("+statusCode+").\n"+
                         "Resource Path = "+unlockMethod.getPath();
        showMessageDialog(title, msg);
      } else  {
        theResultFlag = true;
      }
    } catch(Exception e)  {
      //do nothing
      e.printStackTrace();
    }
    return theResultFlag;
  } //unlock()

  /*
   *  Property Getter
   */
  public static SPProperty getProperty(SPResourceNode resource,
                                       String propertyName) {
    return getProperty(resource.getClient(), resource.getResourceUrlPath(),
                propertyName);
  }

  /*
   *  Property Getter
   */
  public static SPProperty getProperty(HttpClient client,
        String resourceUrlPath, String propertyName) {
    boolean checkParent = false;
    String checkedPath = new String(resourceUrlPath);
    SPPropFindMethod propFindMethod;
    SPProperty theProperty = null;

    Vector propList = new Vector(1);
    propList.add(propertyName);


    do  {

        propFindMethod = new SPPropFindMethod(client);
        propFindMethod.setPath(checkedPath);
	    propFindMethod.setDepth(DepthSupport.DEPTH_0);

        try {
          propFindMethod.executeByName(propList.elements());
          int statusCode = propFindMethod.getStatusCode();
         
          // Handle case of a non-existent file
          // Check permissions on parent
          // Return null for lock property 
          if (statusCode == HttpStatus.SC_NOT_FOUND
              || statusCode == HttpStatus.SC_BAD_REQUEST) {
              if (propertyName.equals("current-user-privilege-set")) { 
                checkParent = (! checkParent) ? true: false; //a toggle
                checkedPath = getParentUrl(checkedPath);
              } else if (propertyName.equals("lockdiscovery"))
                  return null;
          }
          else if ( (statusCode >=200 && statusCode<300) )  {
            checkParent = false;
            theProperty = propFindMethod.getProperty(propertyName);
          } else {
            checkParent = false;
            final String title = "Remote Failure";
            final String msg = "Unable to access the resource properties ("+statusCode+")\n"+
                         "Resource Path = "+resourceUrlPath;
            SPUtils.showMessageDialog(title, msg);
          }

        } catch(Exception e)  {
          //do nothing
          e.printStackTrace();
        }
    } while (checkParent);

    return theProperty;
  }

  /*
   * Property mutator
   */
  public static void setProperty(SPResourceNode resource, 
			Enumeration properties) {
      SPPropPatchMethod pp = new SPPropPatchMethod(resource);
      while(properties.hasMoreElements())
      {
		  WebdavProperty prop = (WebdavProperty)properties.nextElement();
	  	  pp.addPropertyToSet(prop.getName(),
	          encodeXML(prop.getValue()), prop.getNamespace(), 
			  prop.getNamespaceInfo());
      }

      try {
	  pp.execute();
	  int status = pp.getStatusCode();

	  if(!(status>=200 && status<300) &&
	     status != HttpStatus.SC_NOT_FOUND) {
	      SPUtils.showMessageDialog("Remote Failure",
		      "Unable to set property on resource "
		      +resource.getResourceUrlPath()+" ("+status+")");
	  }
      } catch(Exception e) {
	e.printStackTrace();
      }
  }

  /*
   * Checks to see if the passed in path is valid 
   *
   * @Returns false if the resourceUrlPath is a collection or the method fails
   *
   */
  public static boolean isValidUrl(HttpClient client,
                                String resourceUrlPath)  {

    try {
      SPPropFindMethod propFindMethod = new SPPropFindMethod(client);
      propFindMethod.setPath(resourceUrlPath);

      Vector propList = new Vector(2);
      propList.add("resourcetype");
      propList.add("current-user-privilege-set");

      propFindMethod.executeByName(propList.elements());

      int statusCode = propFindMethod.getStatusCode();

      if (!(statusCode>=200 && statusCode<300 )) {
        return false;
      } 

    }catch (Exception e)  {
        return false;
    }
    return true;

  } //getSPResourceNode()


  /*
   * Creates a SPResourceNode object from the given client and resourceUrlPath.
   *
   * @Returns null if the resourceUrlPath is a collection or the method fails
   *
   */
  public static SPResourceNode getSPResourceNode(HttpClient client,
	String resourceUrlPath)  {

    SPResourceNode theSPResourceNode = null;
    try {
      SPPropFindMethod propFindMethod = new SPPropFindMethod(client);
      propFindMethod.setPath(resourceUrlPath);

      Vector propList = new Vector(2);
      propList.add("resourcetype");
      propList.add("current-user-privilege-set");

      propFindMethod.executeByName(propList.elements());

      int statusCode = propFindMethod.getStatusCode();

      if (!(statusCode>=200 && statusCode<300 )) {
        final String title = "Remote Failure";
        final String msg = "Unable to access the remote resource ("
	    +statusCode+") "+resourceUrlPath;
        SPUtils.showMessageDialog(title, msg);
      } else  {
        CurrentUserPrivilegeSetProperty userPrivilege = (CurrentUserPrivilegeSetProperty)
                  propFindMethod.getProperty("current-user-privilege-set");
        boolean readAccess = userPrivilege.hasReadAccess();
        boolean writeAccess = userPrivilege.hasWriteAccess();

	SPProperty p = propFindMethod.getProperty("resourcetype");
	ResourceTypeProperty resourceType = (ResourceTypeProperty)
	    p.getPropertyHandle();

        LockdiscoveryProperty lockdiscoveryProp =
	    (LockdiscoveryProperty)
	     propFindMethod.getProperty("lockdiscovery");

        boolean readOnly=false;
        if (lockdiscoveryProp != null)  { //is being locked
	    String userName = client.getCredentials().getUserName();
	    if (lockdiscoveryProp.getLockToken(userName) == null)  {
	      // is locked by others
	      readOnly = true;
	    }
        }


        if (! resourceType.isCollection())  {
              theSPResourceNode = new SPWebFile(client, resourceUrlPath);
        } else  {
	      theSPResourceNode = new SPWebFolder(client, resourceUrlPath);
	}

	theSPResourceNode.setPrivilege(readAccess, writeAccess);
	theSPResourceNode.setReadOnlyCached(readOnly);
      }

    }catch (Exception e)  {
      e.printStackTrace();
    }
    return theSPResourceNode;

  } //getSPResourceNode()


  private static String encodeXML(String in) {
      StringBuffer buf = new StringBuffer();
      int i;
      char input[] = in.toCharArray();

      for(i=0; i < input.length; i++) {
	  switch(input[i]) {
	      case '&':
		  buf.append("&amp;");
		  break;
	      case '>':
		  buf.append("&gt;");
		  break;
	      case '<':
		  buf.append("&lt;");
		  break;
	      default:
		  buf.append(input[i]);
		  break;
	  }
      }

      return buf.toString();
  }

} //End of SPUtils class










