/*
 * $Header: /home/cvs/jakarta-slide/src/stores/slidestore/cloudscape/CloudscapeDescriptorsStore.java,v 1.2 2001/10/07 19:32:38 dirkv Exp $
 * $Revision: 1.2 $
 * $Date: 2001/10/07 19:32:38 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package slidestore.cloudscape;

import java.sql.*;
import org.apache.slide.common.*;
import org.apache.slide.security.*;
import org.apache.slide.content.*;
import org.apache.slide.util.logger.Logger;
import slidestore.reference.JDBCDescriptorsStore;

/**
 * Cloudscape compliant store implementation.
 * 
 * @author Dirk Verbeeck
 * @version $Revision: 1.2 $
 */
public class CloudscapeDescriptorsStore extends JDBCDescriptorsStore {
    
    /**
     * Returns the sql statements to create the database objects.
     */
    protected String[] getDatabaseCreateStatements()
    {
        String[] statements = {
            "create table objects(uri varchar(3200) primary key, classname varchar(4000))",
            "create table children(uri varchar(4000), childuri varchar(4000))",
            "create table links(link varchar(4000), linkto varchar(4000))",
            "create table permissions(object varchar(4000), revisionnumber varchar(20), " +
            "    subject varchar(4000), \"ACTION\" varchar(4000), inheritable int, negative int)",
            "create table locks(id varchar(4000), object varchar(4000), " +
            "    subject varchar(4000), type varchar(4000), " +
            "    expirationdate varchar(15), inheritable int, xexclusive int)",
            "create table revisions(uri varchar(3200) primary key, " +
            "    isversioned int, initialrevision varchar(10))",
            "create table workingrevision(uri varchar(4000), " +
            "    baserevision varchar(20), xnumber varchar(20))",
            "create table latestrevisions(uri varchar(4000), " +
            "    branchname varchar(4000), xnumber varchar(20))",
            "create table branches(uri varchar(4000), xnumber varchar(20), " +
            "    childnumber varchar(20))",
            "create table revision(uri varchar(4000), xnumber varchar(20), " +
            "    branchname varchar(4000))",
            "create table label(uri varchar(4000), xnumber varchar(20), " +
            "    label varchar(4000))",
            "create table property(uri varchar(4000), xnumber varchar(20), " +
            "    name varchar(4000), value varchar(4000), namespace varchar(4000), " +
            "    type varchar(100), protected int)"};

        return statements;
    }
	
    /**
     * Revoke a permission.
	 * This cloudscape version has quoted the action column because action in a reserved word.
     * 
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Service
     */
    public void revokePermission(Uri uri, NodePermission permission)
        throws ServiceAccessException {
        
        PreparedStatement statement = null;
        
        try {
            NodeRevisionNumber revisionNumber = permission.getRevisionNumber();
            if(revisionNumber != null) {
                statement = connection.prepareStatement
                    ("delete from permissions where object= ? and subject = ? and \"ACTION\" = ?  and revisionnumber = ? ");
                statement.setString(4, revisionNumber.toString());
            }
            else {
                statement = connection.prepareStatement
                    ("delete from permissions where object = ? and subject = ? and \"ACTION\" = ? and revisionnumber is NULL");
            }

            statement.setString(1, permission.getObjectUri());
            statement.setString(2, permission.getSubjectUri());
            statement.setString(3, permission.getActionUri());
        
            statement.execute();
        } catch (SQLException e) {
            getLogger().log(e,LOG_CHANNEL,Logger.ERROR);
            throw new ServiceAccessException(this, e);
        } finally {
            closeStatement(statement);
        }
        
    }
}