/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/methods/OptionsMethod.java,v 1.6 2001/05/01 21:28:01 remm Exp $
 * $Revision: 1.6 $
 * $Date: 2001/05/01 21:28:01 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */
 
package org.apache.webdav.lib.methods;

import java.io.IOException;
import java.io.InputStream;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.State;
import org.apache.commons.httpclient.Header;


/**
 * OPTIONS Method.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 */
public class OptionsMethod
    extends HttpMethodBase {
    
    
    // -------------------------------------------------------------- Constants
    
    
    /**
     * DAV level 1. Mandatory.
     */
    public static final String DAV_LEVEL1 = "1";
    
    
    /**
     * DAV level 2.
     */
    public static final String DAV_LEVEL2 = "2";
    
    
    /**
     * Advanced collections.
     */
    public static final String ADVANCED_COLLECTIONS = "3";
    
    
    /**
     * Delta V.
     */
    public static final String DELTAV = "4";
    
    
    /**
     * ACL.
     */
    public static final String ACL = "5";
    
    
    /**
     * DASL.
     */
    public static final String DASL = "6";
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Method constructor.
     */
    public OptionsMethod() {
        name = "OPTIONS";
    }
    
    
    /**
     * Method constructor.
     */
    public OptionsMethod(String path) {
        super(path);
        name = "OPTIONS";
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * DAV Capabilities.
     */
    private Vector davCapabilities = new Vector();
    
    
    /**
     * Methods allowed.
     */
    private Vector methodsAllowed = new Vector();
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Is the specified method allowed ?
     */
    public boolean isAllowed(String method) {
        checkUsed();
        return methodsAllowed.contains(method);
    }
    
    
    /**
     * Get a list of allowed methods.
     */
    public Enumeration getAllowedMethods() {
        checkUsed();
        return methodsAllowed.elements();
    }
    
    
    /**
     * Is DAV capability supported ?
     */
    public boolean isSupported(String capability) {
        checkUsed();
        return davCapabilities.contains(capability);
    }
    
    
    /**
     * Get a list of supported DAV capabilities.
     */
    public Enumeration getDavCapabilities() {
        checkUsed();
        return davCapabilities.elements();
    }
    
    
    // --------------------------------------------------- WebdavMethod Methods
    
    
    /**
     * Generate the query body.
     *
     * @return String query
     */
    public String generateQuery() {
        return null;
    }
    
    
    /**
     * Parse response.
     *
     * @param is Input stream
     */
    public void parseResponse(InputStream is)
        throws IOException {
    }
    
    
    /**
     * Process response headers. The contract of this method is that it only
     * parses the response headers.
     *
     * @param headers Headers list
     */
    public void processResponseHeaders(Hashtable headers) {
        
        Header davHeader = (Header) headers.get("dav");
        if (davHeader != null) {
            String davHeaderValue = davHeader.getValue();
            StringTokenizer tokenizer =
                new StringTokenizer(davHeaderValue, ",");
            while (tokenizer.hasMoreElements()) {
                String davCapability = tokenizer.nextToken().trim();
                davCapabilities.addElement(davCapability);
            }
        }
        
        Header allowHeader = (Header) headers.get("allow");
        if (allowHeader != null) {
            String allowHeaderValue = allowHeader.getValue();
            StringTokenizer tokenizer =
                new StringTokenizer(allowHeaderValue, ",");
            while (tokenizer.hasMoreElements()) {
                String methodAllowed =
                    tokenizer.nextToken().trim().toUpperCase();
                methodsAllowed.addElement(methodAllowed);
            }
        }
    }
    
    
}
