/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/properties/AclProperty.java,v 1.6 2001/10/12 20:37:44 dirkv Exp $
 * $Revision: 1.6 $
 * $Date: 2001/10/12 20:37:44 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.webdav.lib.properties;

import java.util.Enumeration;
import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import org.apache.util.DOMUtils;

import org.apache.commons.httpclient.URLUtil;

import org.apache.webdav.lib.Ace;
import org.apache.webdav.lib.BaseProperty;
import org.apache.webdav.lib.Privilege;
import org.apache.webdav.lib.ResponseEntity;

/**
 * This interface models the <code>&lt;D:acl&gt;</code> property, which is
 * defined in the WebDAV Access Control Protocol specification.
 *
 * @author Remy Maucherat
 * @author Dirk Verbeeck
 * @version $Revision: 1.6 $
 */
public class AclProperty extends BaseProperty {


    // -------------------------------------------------------------- Constants


    /**
     * The property name.
     */
    public static final String TAG_NAME = "acl";


    // ----------------------------------------------------------- Constructors


    /**
     * Default constructor for the property.
     */
    public AclProperty(ResponseEntity response, Element element) {
        super(response, element);
    }


    // --------------------------------------------------------- Public Methods


    /**
     * Returns the Aces present in this acl property.
     * 
     * @return Ace[] An ace array or null when there is no ace.
     */
    public Ace[] getAces() {
        NodeList children = element.getChildNodes();
        if (children == null || children.getLength() == 0)
            return null;
        ArrayList aces = new ArrayList();
        for (int i = 0; i < children.getLength(); i++) {
            try {
                Element child = (Element) children.item(i);
                String namespace = DOMUtils.getElementNamespaceURI(child);
                if (namespace != null && namespace.equals("DAV:")) {
                    String localName = DOMUtils.getElementLocalName(child);
                    if ("ace".equals(localName)) {
                        aces.add(parseAce(child));
                    }
                }
            } catch (ClassCastException e) {
            }
        }
        return (Ace[]) aces.toArray(new Ace[aces.size()]);
    }


    // ------------------------------------------------------ Protected Methods


    /**
     * Parse an ace.
     */
    protected Ace parseAce(Element element) {

        String principal = null;
        Element child = DOMUtils.getFirstElement(element, "DAV:", "principal");
        if (child == null) {
            System.err.println("Error: mandatory element <principal> is missing !");
            System.err.println("element: " + element);
            return null;
        }
        
        Element href = DOMUtils.getFirstElement(child, "DAV:", "href");
        if (href!=null) 
        {
            principal = URLUtil.URLDecode(DOMUtils.getTextValue(href));
        }

        String[] types={"all","authenticated","unauthenticated","property","self"};
        for (int i=0 ; i<types.length && principal==null ; i++)        
        {
            Element type = DOMUtils.getFirstElement(child, "DAV:", types[i]);
            if (type!=null) 
            {
                principal=types[i];
            }
        }

        if (principal==null) 
        {
            System.err.println("Error: unknown type of principal");
            System.err.println("element: " + element);
            return null;
        }

        Ace ace = new Ace(principal);

        child = DOMUtils.getFirstElement(element, "DAV:", "grant");
        if (child == null) {
            child = DOMUtils.getFirstElement(element, "DAV:", "deny");
            ace.setNegative(true);
         }
        if (child != null) {
            NodeList privilegeElements = child.getElementsByTagNameNS("DAV:", "privilege");
            for (int i = 0; i < privilegeElements.getLength(); i++) {
                Element privilegeElement = (Element) privilegeElements.item(i);
                NodeList privileges = privilegeElement.getElementsByTagName("*");
                for (int j=0 ; j<privileges.getLength() ; j++)
                {
                    Element privilege = (Element) privileges.item(j);
                    ace.addPrivilege(parsePrivilege(privilege));
                }
            }
        }

        child = DOMUtils.getFirstElement(element, "DAV:", "inherited");
        if (child != null) {
            href=DOMUtils.getFirstElement(child, "DAV:", "href");
            if (href!=null)
            {
                ace.setInherited(true);
                ace.setInheritedFrom(DOMUtils.getTextValue(href));
            }
            else 
            {
                System.err.println("Error: mandatory element <href> is missing !");
                return null;
            }
        }
        
        child = DOMUtils.getFirstElement(element, "DAV:", "protected");
        if (child != null) {
            ace.setProtected(true);
        }
        
        return ace;
        
    }


    /**
     * Parse a privilege element.
     */
    protected Privilege parsePrivilege(Element privilegeElement) {
        return new Privilege(privilegeElement.getNamespaceURI(),
                             privilegeElement.getLocalName(), null);
    }

    public String getPropertyAsString() {
        Ace[] aces = getAces();
        
        if ((aces==null) || (aces.length==0))
            return "";
        
        StringBuffer tmp = new StringBuffer(aces[0].toString());
        for (int i=1; i<aces.length ; i++) {
            tmp.append(", ");
            tmp.append(aces[i].toString());
        }

        return tmp.toString();
    }

}
