/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/properties/ResourceTypeProperty.java,v 1.4 2001/08/06 13:26:42 dirkv Exp $
 * $Revision: 1.4 $
 * $Date: 2001/08/06 13:26:42 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.webdav.lib.properties;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.apache.util.DOMUtils;

import org.apache.webdav.lib.BaseProperty;
import org.apache.webdav.lib.ResponseEntity;

/**
 * An interface that describes a standard Resource Type property (as defined by
 * the WebDAV specification).
 *
 * @author <a href="mailto:bcholmes@interlog.com">B.C. Holmes</a>
 * @author Dirk Verbeeck
 */
public class ResourceTypeProperty extends BaseProperty {

    private boolean initialized=false;
    private boolean isCollection;
    
    // -------------------------------------------------------------- Constants


    /**
     * The property name.
     */
    public static final String TAG_NAME = "resourcetype";


    // ----------------------------------------------------------- Constructors


    /**
     * Default constructor for the property.
     */
    public ResourceTypeProperty(ResponseEntity response, Element element) {
        super(response, element);
    }


    // --------------------------------------------------------- Public Methods


    /**
     * Returns true if the resource is a collection.  A collection is indicated
     * by a response like this:
     *
     * <pre>
     * &lt;D:resourcetype&gt;&lt;D:collection/&gt;&lt;/D:resourcetype&gt;
     * </pre>
     */
    public boolean isCollection() {
        init();
        return isCollection;
    }

    private void init() 
    {
        // FIXME: only <DAV:collection/> is supported
    
        if (initialized)
            return;
            
        NodeList tmp = element.getChildNodes();
        for (int i = 0; tmp != null && i < tmp.getLength(); i++ ) {
            try {
                Element child = (Element) tmp.item(i);
                if ("collection".equals(DOMUtils.getElementLocalName(child)) 
                     && "DAV:".equals(DOMUtils.getElementNamespaceURI(child)))
                {
                    isCollection=true;
                    initialized=true;
                    return;
                }
            } catch (ClassCastException e) {
            }
        }
        isCollection=false;
        initialized=true;
    }

    /**
     * This method returns the value of the property.
     * For this property "COLLECTION" is returned if
     * this resource is a collection, "" otherwise.
     * 
     * WARNING: this will change in the future
     * use isCollection()
     */
    public String getPropertyAsString() {
        init();
        return isCollection?"COLLECTION":"";
    }
}
