/*
 * $Header: /home/cvs/jakarta-slide/src/wrappers/catalina/SlideLogger.java,v 1.1 2001/09/08 20:03:12 remm Exp $
 * $Revision: 1.1 $
 * $Date: 2001/09/08 20:03:12 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package wrappers.catalina;

import java.util.Date;
import java.util.Locale;
import java.text.SimpleDateFormat;

import org.apache.catalina.Logger;

/**
 * Simple logger implementation.
 *
 * @author Remy Maucherat
 */
public class SlideLogger extends org.apache.slide.util.logger.SimpleLogger
    implements org.apache.slide.util.logger.Logger {
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Catalina logger.
     */
    protected Logger logger;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Set Catalina logger.
     */
    public void setLogger(Logger logger) {
        this.logger = logger;
    }


    /**
     * Get Catalina logger.
     */
    public Logger getLogger() {
        return logger;
    }
    
    
    // --------------------------------------------------------- Logger Methods
    
    
    /**
     * Log an object thru the specified channel and with the specified level.
     *
     * @param data The object to log.
     * @param channel The channel name used for logging.
     * @param level The level used for logging.
     */
    public void log(Object data, String channel, int level) {

        if (isEnabled(channel, level)) {

            String levelValue = "";
            if (channel.equals(DEFAULT_CHANNEL))
                channel = "";
            else
                channel = channel + " - ";
            if ((level >= 0) && (level < loggingLevels.length))
                levelValue = loggingLevels[level];

            String message = null;
            if (dateFormat == null) {
                message = System.currentTimeMillis() + " - "
                    + channel + levelValue + " - " + data;
            } else {
                message = dateFormat.format(new Date()) + " - "
                    + channel + levelValue + " - " + data;
            }
            
            if (data instanceof Throwable) {
                logger.log(message, (Throwable) data);
            } else {
                logger.log(message);
            }

        }

    }
    
    
}
